@@ -33,7 +33,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <assert.h>
 
 #if USE_POSIX_THREADS
 # include <pthread.h>
@@ -90,8 +89,9 @@ use_pthread_p (void)
           void *retval;
           if (pthread_join (thread, &retval) != 0)
             {
-              assert (!"pthread_join");
-              abort ();
+              fputs ("gpgrt fatal: pthread_join in use_pthread_p failed\n",
+                     stderr);
+              _gpgrt_abort ();
             }
           result = 1;
         }
@@ -110,13 +110,13 @@ get_lock_object (gpgrt_lock_t *lockhd)
 
   if (lock->vers != LOCK_ABI_VERSION)
     {
-      assert (!"lock ABI version");
-      abort ();
+      fputs ("gpgrt fatal: lock ABI version mismatch\n", stderr);
+      _gpgrt_abort ();
     }
   if (sizeof (gpgrt_lock_t) < sizeof (_gpgrt_lock_t))
     {
-      assert (!"sizeof lock obj");
-      abort ();
+      fputs ("gpgrt fatal: sizeof lock obj\n", stderr);
+      _gpgrt_abort ();
     }
 
   return lock;
@@ -136,8 +136,8 @@ _gpgrt_lock_init (gpgrt_lock_t *lockhd)
     {
       if (sizeof (gpgrt_lock_t) < sizeof (_gpgrt_lock_t))
         {
-          assert (!"sizeof lock obj");
-          abort ();
+          fputs ("gpgrt fatal: sizeof lock obj\n", stderr);
+          _gpgrt_abort ();
         }
       lock->vers = LOCK_ABI_VERSION;
     }
